/* ######################################################################### */
/* ##                                                                     ## */
/* ##  U T I L / U N I X C F G P A R S E R . J A V A                      ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Job .......: Parse a Unix Configuration file from an instance      ## */
/* ##               of java.io.BufferedReader (or its subclasses)         ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Copyright (C) 2002  Daniel Scheibli                                ## */
/* ##                                                                     ## */
/* ##  This library is free software; you can redistribute it and/or      ## */
/* ##  modify it under the terms of the GNU Lesser General Public         ## */
/* ##  License as published by the Free Software Foundation; either       ## */
/* ##  version 2.1 of the License, or (at your option) any later version. ## */
/* ##                                                                     ## */
/* ##  This library is distributed in the hope that it will be useful,    ## */
/* ##  but WITHOUT ANY WARRANTY; without even the implied warranty of     ## */
/* ##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  ## */
/* ##  Lesser General Public License for more details.                    ## */
/* ##                                                                     ## */
/* ##  You should have received a copy of the GNU Lesser General Public   ## */
/* ##  License along with this library; if not, write to the Free         ## */
/* ##  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,     ## */
/* ##  MA 02111-1307 USA                                                  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Remark ....: <none>                                                ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Author ....: Daniel Scheibli <daniel.scheibli@edelbyte.org>        ## */
/* ##  Date ......: 2002-11-10                                            ## */
/* ##  Changes ...: ----- UnixCfgParser.java V00.99/R020 ---------------- ## */
/* ##               <none>                                                ## */
/* ##               ----- UnixCfgParser.java V00.99/R011 ---------------- ## */
/* ##               <none>                                                ## */
/* ##                                                                     ## */
/* ######################################################################### */



/* ##### Presettings */
/* ------------------------------------------------------------------------- */
package org.edelbyte.util;
/* ------------------------------------------------------------------------- */





/* ##### Class definition */
/** ************************************************************************
 * A basic Unix configuration file parser. The parser reads from
 * a <code>java.io.BufferedReader</code> (or one of its subclasses)
 * and returns a <code>java.util.Properties</code> object with
 * the found key and value pairs. It is an alternative to the
 * <code>java.lang.Properties.load()</code> method (which
 * has some strange behavior).<br />
 * <br />
 * More information can be found in the detailed description
 * of the {@link #parse} method.
 *
 * @author Daniel Scheibli &lt;daniel.scheibli at edelbyte.org&gt;
 * ************************************************************************* */
public class UnixCfgParser
{
 /* ##### Presettings (class fields) */
 /* ------------------------------------------------------------------------ */
 /** ***********************************************************************
  * Constant holding the token used for the comment detection
  * within the stream.
  * ************************************************************************ */
 protected static final java.lang.String COMMENT_TOKEN      = "#";
 /** ***********************************************************************
  * Constant holding the token used for the key and value pair
  * separation within the stream.
  * ************************************************************************ */
 protected static final java.lang.String EQUALS_TOKEN       = "=";
 /** ***********************************************************************
  * Constant holding the token used for the line continuation detection
  * within the stream.
  * ************************************************************************ */
 protected static final java.lang.String CONTINUATION_TOKEN = " \\"; 
 /* ------------------------------------------------------------------------ */





 /* ##### Presettings (instance fields) */
 /* ------------------------------------------------------------------------ */
 /** ***********************************************************************
  * Field holding the token, which indicates the beginning of the
  * key and value pairs part of the stream. It is used for the
  * Between Check mechanism.
  * ************************************************************************ */
 protected java.lang.String oBetweenBeginToken   = "#BEGIN";
 /** ***********************************************************************
  * Field holding the token, which indicates the end of the
  * key and value pairs part of the stream. It is used for the
  * Between Check mechanism.
  * ************************************************************************ */
 protected java.lang.String oBetweenEndToken     = "#END";
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -- */
 /** ***********************************************************************
  * Field holding the activation status of the Between Check
  * mechanism (default is <code>false</code>).
  * ************************************************************************ */
 protected boolean          bBetweenCheckEnabled = false;
 /* ------------------------------------------------------------------------ */ 





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   U N I X C F G P A R S E R ()                                     ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Creates a new instance of this class.
  * ************************************************************************ */
 public UnixCfgParser( )
 { }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   G E T B E T W E E N B E G I N T O K E N ()                       ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Returns the token, which indicates the beginning of the
  * key and value pairs part
  * (from the {@link #oBetweenBeginToken} field).
  *
  * @return A new String, containing the indicator token
  * ************************************************************************ */ 
 public java.lang.String getBetweenBeginToken( )
 { return( this.oBetweenBeginToken ); }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   G E T B E T W E E N E N D T O K E N ()                           ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Returns the token, which indicates the end of the
  * key and value pairs part
  * (from the {@link #oBetweenEndToken} field).
  *
  * @return A new String, containing the indicator token
  * ************************************************************************ */ 
 public java.lang.String getBetweenEndToken( )
 { return( this.oBetweenEndToken ); }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   I S B E T W E E N C H E C K E N A B L E D ()                     ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Returns <code>true</code> for the case, that the Between Check
  * mechanism is active.<br />
  * <br />
  * The Between Check mechanism ensures, that there are key and value
  * pairs between the so called BetweenBegin and BetweenEnd tokens
  * (see {@link #getBetweenBeginToken} and {@link #getBetweenEndToken})
  * only.
  *
  * @return <code>true</code> if the Between Check mechanism
  * is active.
  * ************************************************************************ */ 
 public boolean isBetweenCheckEnabled( )
 { return( this.bBetweenCheckEnabled ); }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   P A R S E ()                                                     ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Reads key and value pairs from a <code>java.io.BufferedReader</code>
  * (or one of its subclasses).<br />
  * <br />
  * The parser reads the stream line wise using the <code>readLine()</code>
  * method of the object given by the <code>oReader</code> parameter.
  * A line can hold up to one key and value pair. Everything after the
  * first occurrence of &quot;<code>#</code>&quot; is interpreted as
  * comment and therefore skipped. Lines containing tabs and blanks only
  * are skipped as well. The first occurrence of &quot;<code>=</code>&quot;
  * is interpreted as a delimiter separating key and value. After the
  * separation of key and value, the key is converted to upper case.
  * Lines ending with &quot;<code> \</code>&quot; (please notice the
  * blank leading the backslash!) indicates a line continuation.<br />
  * <br />
  * An sample configuration file is listed below:<br />
  * <br />
  * <tt>##### Database configuration file for SAMPLE database<br />
  *     <br />
  *     #BEGIN<br />
  *     Backup.Enabled = YES&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;##### Backup allowed [YES|NO]<br />
  *     Backup.Rule.1&#160;&#160;= (A &gt; B) &amp;&amp;&#160;&#160;\&#160;&#160;&#160;##### Backup rule number one<br />
  *     &#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160;(C != D)<br />
  *     #END<br /></tt>
  * <br />
  * For the given example, the returning <code>java.util.Properties</code>
  * object contains the following key and value pairs:<br />
  * <ul>
  *  <li>The key is &quot;<code>BACKUP.ENABLED</code>&quot; and the associated
  *      value is &quot;<code>YES</code>&quot;.</li>
  *  <li>The key is &quot;<code>BACKUP.RULE.1</code>&quot; and the associated
  *      value is &quot;<code>(A &gt; B) && (C != D)</code>&quot;.</li>
  * </ul>
  *
  * @param oReader
  *        The name of the DB2 Instance to be addressed.
  * @return The <code>java.util.Properties</code> object containing
  *         the key and value pairs found while parsing.
  * @exception java.io.IOException
  *            For the case, that there was an error while reading
  *            the stream.
  * @exception java.text.ParseException
  *            For the case, that there was an error while parsing
  *            the stream.
  * ************************************************************************ */ 
 public java.util.Properties parse( java.io.BufferedReader oReader ) throws java.io.IOException,
                                                                            java.text.ParseException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  java.util.Properties   oPairTable          = new java.util.Properties();  
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  boolean                bIsBetweenFlag;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */  
  java.lang.String       oCurrentRow;
  java.lang.String       oCurrentRowKey      = null;
  java.lang.String       oCurrentRowValue;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  int                    iRowNumber          = -1;
  int                    iRowNumberValueNext = -1;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  int                    iDummy;
  /* ----------------------------------------------------------------------- */
  


  /* ##### Initialize the Between Check related stuff */
  /* ----------------------------------------------------------------------- */
  if( bBetweenCheckEnabled == true )
  { bIsBetweenFlag = false; }
  else
  { bIsBetweenFlag = true; }
  /* ----------------------------------------------------------------------- */
  


  /* ##### Loop over the stream */
  /* ----------------------------------------------------------------------- */
  while( (oCurrentRow = oReader.readLine()) != null )
  {
   /* ##### Presettings */
   iRowNumber++;   

   /* ##### BetweenCheck: Detect the indicators */
   /* ##### (given the fact, that the check is enabled) */
   if( bBetweenCheckEnabled == true )
   {
    if( oCurrentRow.trim().equals( oBetweenBeginToken ) == true )
    { bIsBetweenFlag = true;  continue; }
    if( oCurrentRow.trim().equals( oBetweenEndToken )   == true )
    { bIsBetweenFlag = false; continue; }   
   }

   /* ##### Remove the comment part from the current row */
   iDummy = oCurrentRow.indexOf( COMMENT_TOKEN );
   if( iDummy > -1 )
   { oCurrentRow = oCurrentRow.substring( 0, iDummy ); }

   /* ##### Skip empty rows */ 
   if( oCurrentRow.trim().length() == 0 )
   { continue; }
   
   /* ##### BetweenCheck: If there is content,... */
   /* ##### (given the fact, that the check is enabled) */
   if( (bBetweenCheckEnabled == true) && (bIsBetweenFlag == false) )
   { throw new java.text.ParseException( "The Between Check feature found content outside the allowed boundaries (RowNo = " + iRowNumber + ").", iRowNumber ); }

   /* ##### Check integrity (based on line continuation only) */
   if( (iRowNumberValueNext >= 0) && (iRowNumberValueNext != iRowNumber) )
   { throw new java.text.ParseException( "A line continuation was indicated, but seems to be interrupted (RowNo = " + iRowNumber + ").", iRowNumber ); }

   /* ##### Seperate the current row into (Key and) Value part */
   if( iRowNumberValueNext < 0 )
   {
    /* ##### Detect the first equals character (our delimiter) */
    iDummy = oCurrentRow.indexOf( EQUALS_TOKEN );
    if( iDummy > -1 )
    {
     oCurrentRowValue = ("A" + oCurrentRow.substring( iDummy + 1 )).trim();
     oCurrentRowKey   =        oCurrentRow.substring( 0, iDummy ).trim().toUpperCase();
     /* ##### On error,... */
     if( oCurrentRowKey.length() == 0 )
     { throw new java.text.ParseException( "Invalid Key part within the current Key/Value pair found (RowNo = " + iRowNumber + ").", iRowNumber ); }
    }
    /* ##### On error,... */
    else
    { throw new java.text.ParseException( "A Key/Value pair was expected, but there was no equals/delimiter found (RowNo = " + iRowNumber + ").", iRowNumber ); }
   }
   else
   { oCurrentRowValue = ("A" + oCurrentRow).trim(); }

   /* ##### Process tailing Continuation Token (indicates line continuation) */
   if( oCurrentRowValue.endsWith( CONTINUATION_TOKEN ) == true )
   {
    oCurrentRowValue    = oCurrentRowValue.substring( 0, oCurrentRowValue.length() - CONTINUATION_TOKEN.length() );
    iRowNumberValueNext = iRowNumber + 1;
   }
   else
   { iRowNumberValueNext = -1; }

   /* ##### Remove the magic "A" and temporary append the magic "Z" */
   /* ##### to remove the leading [tabs|blanks] */
   oCurrentRowValue = (oCurrentRowValue.substring( 1 ) + "Z").trim();
   oCurrentRowValue = oCurrentRowValue.substring( 0, oCurrentRowValue.length() - 1 );
   
   /* ##### Add the Key/Value pairs to the Hashtable */
   if( oPairTable.get( oCurrentRowKey ) != null )
   { oPairTable.put( oCurrentRowKey, oPairTable.get(oCurrentRowKey).toString() + oCurrentRowValue ); }
   else
   { oPairTable.put( oCurrentRowKey, oCurrentRowValue ); }
  }
  /* ----------------------------------------------------------------------- */



  /* ##### Check integrity (based on "open" line continuation) */
  /* ----------------------------------------------------------------------- */
  if( iRowNumberValueNext >= 0 )
   { throw new java.text.ParseException( "A line continuation was indicated, but there was no line continuation found (RowNo = " + iRowNumber + ").", iRowNumber ); }
  /* ----------------------------------------------------------------------- */



  /* ##### Return the Key/Value pairs */
  /* ----------------------------------------------------------------------- */  
  return( oPairTable );
  /* ----------------------------------------------------------------------- */
 }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   S E T B E T W E E N B E G I N T O K E N ()                       ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Set a alternative value for the BetweenBegin token, which is
  * used by the Between Check mechanism.
  *
  * @param oBetweeenBeginToken
  *        The String containing the new value
  *        (for the {@link #oBetweenBeginToken} field).
  * ************************************************************************ */ 
 public void setBetweenBeginToken( java.lang.String oBetweenBeginToken )
 { this.oBetweenBeginToken = oBetweenBeginToken; }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   S E T B E T W E E N C H E C K E N A B L E D ()                   ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Enables or disables the Between Check mechanism while parsing the
  * stream.<br />
  * <br />
  * The Between Check mechanism ensures, that there are key and value
  * pairs between the so called BetweenBegin and BetweenEnd tokens
  * (see {@link #getBetweenBeginToken} and {@link #getBetweenEndToken})
  * only.
  *
  * @param bStatus
  *        The status for the Between Check mechanism to be set.
  * ************************************************************************ */ 
 public void setBetweenCheckEnabled( boolean bStatus )
 { this.bBetweenCheckEnabled = bStatus; }





 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   S E T B E T W E E N E N D T O K E N ()                           ## */
 /* ##                                                                    ## */ 
 /* ######################################################################## */
 /** ***********************************************************************
  * Set a alternative value for the BetweenEnd token, which is
  * used by the Between Check mechanism.
  *
  * @param oBetweeenEndToken
  *        The String containing the new value
  *        (for the {@link #oBetweenEndToken} field).
  * ************************************************************************ */ 
 public void setBetweenEndToken( java.lang.String oBetweenEndToken )
 { this.oBetweenEndToken = oBetweenEndToken; }
}
/* ------------------------------------------------------------------------- */




